define(["require", "exports", "common/alertArgs", "common/alertType", "common/appUrl"], function(require, exports, alertArgs, alertType, appUrl) {
    var commandBase = (function () {
        function commandBase() {
        }
        commandBase.prototype.execute = function () {
            throw new Error("Execute must be overridden.");
        };

        commandBase.prototype.reportInfo = function (title, details) {
            this.reportProgress(0 /* info */, title, details);
        };

        commandBase.prototype.reportError = function (title, details, httpStatusText) {
            this.reportProgress(3 /* danger */, title, details, httpStatusText);
            if (console && console.log && typeof console.log === "function") {
                console.log("Error during command execution", title, details, httpStatusText);
            }
        };

        commandBase.prototype.reportSuccess = function (title, details) {
            this.reportProgress(1 /* success */, title, details);
        };

        commandBase.prototype.reportWarning = function (title, details, httpStatusText) {
            this.reportProgress(2 /* warning */, title, details, httpStatusText);
        };

        commandBase.prototype.urlEncodeArgs = function (args) {
            var propNameAndValues = [];
            for (var prop in args) {
                var value = args[prop];
                if (value instanceof Array) {
                    for (var i = 0; i < value.length; i++) {
                        propNameAndValues.push(prop + "=" + encodeURIComponent(value[i]));
                    }
                } else if (value !== undefined) {
                    propNameAndValues.push(prop + "=" + encodeURIComponent(value));
                }
            }

            return "?" + propNameAndValues.join("&");
        };

        commandBase.prototype.query = function (relativeUrl, args, resource, resultsSelector) {
            var ajax = this.ajax(relativeUrl, args, "GET", resource);
            if (resultsSelector) {
                var task = $.Deferred();
                ajax.done(function (results, status, xhr) {
                    var transformedResults = resultsSelector(results);
                    task.resolve(transformedResults);
                });
                ajax.fail(function (request, status, error) {
                    task.reject(request, status, error);
                });
                return task;
            } else {
                return ajax;
            }
        };

        commandBase.prototype.head = function (relativeUrl, args, resource, resultsSelector) {
            var ajax = this.ajax(relativeUrl, args, "HEAD", resource);
            if (resultsSelector) {
                var task = $.Deferred();
                ajax.done(function (results, status, xhr) {
                    var allHeaders = xhr.getAllResponseHeaders();
                    if (allHeaders) {
                        var headersObject = {};
                        var headersArray = xhr.getAllResponseHeaders().trim().split(/\r?\n/);
                        for (var n = 0; n < headersArray.length; n++) {
                            var keyValue = headersArray[n].split(": ");
                            if (keyValue.length == 2) {
                                keyValue[1] = keyValue[1].replaceAll("\"", "");
                                headersObject[keyValue[0]] = keyValue[1];
                            }
                        }
                        var transformedResults = resultsSelector(headersObject);
                        task.resolve(transformedResults);
                    }
                });
                ajax.fail(function (request, status, error) {
                    task.reject(request, status, error);
                });
                return task;
            } else {
                return ajax;
            }
        };

        commandBase.prototype.put = function (relativeUrl, args, resource, options) {
            return this.ajax(relativeUrl, args, "PUT", resource, options);
        };

        commandBase.prototype.reset = function (relativeUrl, args, resource, options) {
            return this.ajax(relativeUrl, args, "RESET", resource, options);
        };

        commandBase.prototype.del = function (relativeUrl, args, resource, options) {
            return this.ajax(relativeUrl, args, "DELETE", resource, options);
        };

        commandBase.prototype.post = function (relativeUrl, args, resource, options) {
            return this.ajax(relativeUrl, args, "POST", resource, options);
        };

        commandBase.prototype.ajax = function (relativeUrl, args, method, resource, options) {
            var contentType = method === "GET" ? "text/plain; charset=utf-8" : "application/json; charset=utf-8";
            var defaultOptions = {
                cache: false,
                url: appUrl.forResourceQuery(resource) + relativeUrl,
                data: args,
                dataType: "json",
                contentType: contentType,
                type: method,
                headers: undefined
            };

            if (options) {
                for (var prop in options) {
                    defaultOptions[prop] = options[prop];
                }
            }

            return $.ajax(defaultOptions);
        };

        commandBase.prototype.reportProgress = function (type, title, details, httpStatusText) {
            ko.postbox.publish("Alert", new alertArgs(type, title, details));
        };
        commandBase.ravenClientVersion = '3.0.0.0';
        return commandBase;
    })();

    
    return commandBase;
});
