define(["require", "exports", "durandal/composition", "ace/ace"], function(require, exports, composition, ace) {
    var aceEditorBindingHandler = (function () {
        function aceEditorBindingHandler() {
            this.defaults = {
                theme: "ace/theme/github",
                fontSize: "16px",
                lang: "ace/mode/csharp",
                readOnly: false
            };
        }
        aceEditorBindingHandler.install = function () {
            if (!ko.bindingHandlers["aceEditor"]) {
                ko.bindingHandlers["aceEditor"] = new aceEditorBindingHandler();

                composition.addBindingHandler("aceEditor");
            }
        };

        aceEditorBindingHandler.prototype.init = function (element, valueAccessor, allBindings, viewModel, bindingContext) {
            var bindingValues = valueAccessor();
            var theme = bindingValues.theme || this.defaults.theme;
            var fontSize = bindingValues.fontSize || this.defaults.fontSize;
            var lang = bindingValues.lang || this.defaults.lang;
            var readOnly = bindingValues.readOnly || this.defaults.readOnly;
            var code = typeof bindingValues.code === "function" ? bindingValues.code : bindingContext.$rawData;

            if (typeof code !== "function") {
                throw new Error("code should be an observable");
            }

            var aceEditor = ace.edit(element);
            aceEditor.setTheme(theme);
            aceEditor.setFontSize(fontSize);
            aceEditor.getSession().setMode(lang);
            aceEditor.setReadOnly(readOnly);

            var aceFocusElement = ".ace_text-input";
            $(element).on('blur', aceFocusElement, function () {
                return code(aceEditor.getSession().getValue());
            });

            ko.utils.domNodeDisposal.addDisposeCallback(element, function () {
                return $(element).off('blur', aceFocusElement);
            });

            ko.utils.domData.set(element, "aceEditor", aceEditor);

            if (bindingValues.getFocus && bindingValues.getFocus == true) {
                aceEditor.focus();
            }
        };

        aceEditorBindingHandler.prototype.update = function (element, valueAccessor, allBindings, viewModel, bindingContext) {
            var bindingValues = valueAccessor();
            var code = ko.unwrap(bindingValues.code);
            var aceEditor = ko.utils.domData.get(element, "aceEditor");
            var editorCode = aceEditor.getSession().getValue();
            if (code !== editorCode) {
                aceEditor.getSession().setValue(code);
            }
        };
        return aceEditorBindingHandler;
    })();

    
    return aceEditorBindingHandler;
});
