define(["require", "exports", "common/alertType"], function(require, exports, alertType) {
    var alertArgs = (function () {
        function alertArgs(type, title, details, httpStatusText) {
            if (typeof details === "undefined") { details = ""; }
            if (typeof httpStatusText === "undefined") { httpStatusText = ""; }
            this.type = type;
            this.title = title;
            this.details = details;
            this.httpStatusText = httpStatusText;
            var hashString = (title + details).hashCode().toString();

            this.id = "alert_" + hashString;
        }
        Object.defineProperty(alertArgs.prototype, "errorMessage", {
            get: function () {
                var error = this.errorInfo;
                if (error && error.message) {
                    return error.message;
                }

                return null;
            },
            enumerable: true,
            configurable: true
        });

        Object.defineProperty(alertArgs.prototype, "errorInfo", {
            get: function () {
                if (this.parsedErrorInfo) {
                    return this.parsedErrorInfo;
                }

                if (this.type !== 3 /* danger */ && this.type !== 2 /* warning */) {
                    return null;
                }

                var detailsObj = this.getDetailsObject();
                if (detailsObj) {
                    var error = detailsObj.Error;
                    if (error && typeof error === "string") {
                        var indexOfStackTrace = error.indexOf("\r\n");

                        if (indexOfStackTrace !== -1) {
                            this.parsedErrorInfo = {
                                message: detailsObj.Message ? detailsObj.Message : error.substr(0, indexOfStackTrace),
                                stackTrace: detailsObj.Message ? error : error.substr(indexOfStackTrace + "\r\n".length),
                                url: detailsObj.Url || ""
                            };
                        } else {
                            this.parsedErrorInfo = {
                                message: detailsObj.Message ? detailsObj.Message : error,
                                stackTracke: error,
                                url: detailsObj.Url
                            };
                        }
                    }
                }

                return this.parsedErrorInfo;
            },
            enumerable: true,
            configurable: true
        });

        alertArgs.prototype.getDetailsObject = function () {
            if (this.detailsObject) {
                return this.detailsObject;
            }

            if (this.details) {
                try  {
                    this.detailsObject = JSON.parse(this.details);
                } catch (error) {
                    return null;
                }
            }

            return this.detailsObject;
        };
        return alertArgs;
    })();

    
    return alertArgs;
});
