define(["require", "exports", "durandal/composition"], function(require, exports, composition) {
    var dynamicHeightBindingHandler = (function () {
        function dynamicHeightBindingHandler() {
            var _this = this;
            this.throttleTimeMs = 100;
            var $window = $(window);
            this.windowHeightObservable = ko.observable($window.height());
            window['ravenStudioWindowHeight'] = this.windowHeightObservable.throttle(this.throttleTimeMs);
            $window.resize(function (ev) {
                return _this.windowHeightObservable($window.height());
            });
        }
        dynamicHeightBindingHandler.install = function () {
            if (!ko.bindingHandlers["dynamicHeight"]) {
                ko.bindingHandlers["dynamicHeight"] = new dynamicHeightBindingHandler();

                composition.addBindingHandler('dynamicHeight');
            }
        };

        dynamicHeightBindingHandler.prototype.init = function (element, valueAccessor, allBindings, viewModel, bindingContext) {
            if (valueAccessor().target) {
                element.style.overflowX = "hidden";
            }
        };

        dynamicHeightBindingHandler.prototype.update = function (element, valueAccessor, allBindings, viewModel, bindingContext) {
            var bindingValue = valueAccessor();
            if (bindingValue.target) {
                var newWindowHeight = bindingValue.resizeTrigger;
                var targetSelector = bindingValue.target || "footer";
                var bottomMargin = bindingValue.bottomMargin || 0;

                var lastWindowHeightKey = "ravenStudioLastDispatchedHeight";
                var lastWindowHeight = ko.utils.domData.get(element, lastWindowHeightKey);
                if (lastWindowHeight !== newWindowHeight) {
                    ko.utils.domData.set(element, lastWindowHeightKey, newWindowHeight);
                    this.stickToTarget(element, targetSelector, bottomMargin);
                }
            }
        };

        dynamicHeightBindingHandler.prototype.stickToTarget = function (element, targetSelector, bottomMargin) {
            var targetElement = $(targetSelector);
            if (targetSelector.length === 0) {
                throw new Error("Couldn't configure dynamic height because the target element isn't on the page. Target element: " + targetSelector);
            }

            var $element = $(element);
            var isVisible = $element.is(":visible");

            if (isVisible) {
                var elementTop = $element.offset().top;
                var footerTop = $(targetSelector).position().top;
                var padding = 5 + bottomMargin;
                var desiredElementHeight = footerTop - elementTop - padding;
                var minimumHeight = 100;
                if (desiredElementHeight >= minimumHeight) {
                    $element.height(desiredElementHeight);
                    $element.trigger("DynamicHeightSet", desiredElementHeight);
                }
            }
        };
        return dynamicHeightBindingHandler;
    })();

    
    return dynamicHeightBindingHandler;
});
