define(["require", "exports"], function(require, exports) {
    var databaseAccess = (function () {
        function databaseAccess(dto) {
            var _this = this;
            this.admin = ko.observable();
            this.tenantId = ko.observable();
            this.readOnly = ko.observable();
            this.currentAccessType = ko.computed({
                read: function () {
                    if (_this.admin() === true) {
                        return databaseAccess.adminAccessType;
                    } else if (_this.readOnly() === true) {
                        return databaseAccess.readOnlyAccessType;
                    }

                    return databaseAccess.readWriteAccessType;
                },
                write: function (value) {
                    switch (value) {
                        case databaseAccess.adminAccessType:
                            _this.admin(true);
                            _this.readOnly(false);
                            break;
                        case databaseAccess.readOnlyAccessType:
                            _this.admin(false);
                            _this.readOnly(true);
                            break;
                        case databaseAccess.readWriteAccessType:
                            _this.admin(false);
                            _this.readOnly(false);
                            break;
                        default:
                    }
                }
            });
            this.admin(dto.Admin);
            this.readOnly(dto.ReadOnly);
            this.tenantId(dto.TenantId);

            this.tenantIdOrDefault = ko.computed(function () {
                return _this.tenantId() ? _this.tenantId() : "Select a database";
            });
        }
        databaseAccess.prototype.toDto = function () {
            return {
                Admin: this.admin(),
                TenantId: this.tenantId(),
                ReadOnly: this.readOnly()
            };
        };

        databaseAccess.empty = function () {
            return new databaseAccess({
                Admin: false,
                TenantId: null,
                ReadOnly: false
            });
        };

        databaseAccess.prototype.getTypes = function () {
            return databaseAccess.databaseAccessTypes();
        };
        databaseAccess.adminAccessType = 'Admin';
        databaseAccess.readWriteAccessType = 'Read,Write';
        databaseAccess.readOnlyAccessType = 'Read Only';
        databaseAccess.databaseAccessTypes = ko.observableArray([databaseAccess.adminAccessType, databaseAccess.readWriteAccessType, databaseAccess.readOnlyAccessType]);
        return databaseAccess;
    })();
    
    return databaseAccess;
});
