define(["require", "exports"], function(require, exports) {
    var querySort = (function () {
        function querySort() {
            var _this = this;
            this.fieldName = ko.observable();
            this.sortDirection = ko.observable(0);
            this.isRange = ko.observable(false);
            this.isAscending = ko.observable(true);
            this.fieldNameOrDefault = ko.computed(function () {
                return _this.fieldName() ? _this.fieldName() : "Select a field";
            });
        }
        querySort.prototype.toggleAscending = function () {
            this.isAscending.toggle();
        };

        querySort.prototype.toggleRange = function () {
            this.isRange.toggle();
        };

        querySort.prototype.toQuerySortString = function () {
            if (this.isRange() === true) {
                if (this.isAscending() === true) {
                    return this.fieldName() + querySort.rangeIndicator;
                } else {
                    return "-" + this.fieldName() + querySort.rangeIndicator;
                }
            } else {
                if (this.isAscending() === true) {
                    return this.fieldName();
                } else {
                    return "-" + this.fieldName();
                }
            }
        };

        querySort.prototype.toHumanizedString = function () {
            if (this.isRange() === true) {
                if (this.isAscending() === true) {
                    return this.fieldName() + " range";
                } else {
                    return this.fieldName() + " range descending";
                }
            } else {
                if (this.isAscending() === true) {
                    return this.fieldName();
                } else {
                    return this.fieldName() + " descending";
                }
            }
        };

        querySort.fromQuerySortString = function (querySortText) {
            var sortDirection = 0;
            var sortField = "";

            var isDescending = querySortText.slice(0, 1) === "-";
            var isRange = querySortText.length > querySort.rangeIndicator.length && querySortText.substr(querySortText.length - querySort.rangeIndicator.length) === querySort.rangeIndicator;
            if (isRange && isDescending) {
                sortField = querySortText.substr(1, querySortText.length - 1 - querySort.rangeIndicator.length);
            } else if (isRange && !isDescending) {
                sortField = querySortText.substr(0, querySortText.length - 1 - querySort.rangeIndicator.length);
            } else if (!isRange && isDescending) {
                sortField = querySortText.substr(1, querySortText.length - 1);
            } else {
                sortField = querySortText;
            }

            var q = new querySort();
            q.isAscending(!isDescending);
            q.isRange(isRange);
            q.fieldName(sortField);
            return q;
        };
        querySort.rangeIndicator = "_Range";
        return querySort;
    })();

    
    return querySort;
});
