define(["require", "exports"], function(require, exports) {
    var replicationDestination = (function () {
        function replicationDestination(dto) {
            var _this = this;
            this.url = ko.observable().extend({ required: true });
            this.username = ko.observable().extend({ required: true });
            this.password = ko.observable().extend({ required: true });
            this.domain = ko.observable().extend({ required: true });
            this.apiKey = ko.observable().extend({ required: true });
            this.database = ko.observable().extend({ required: true });
            this.transitiveReplicationBehavior = ko.observable().extend({ required: true });
            this.ignoredClient = ko.observable().extend({ required: true });
            this.disabled = ko.observable().extend({ required: true });
            this.clientVisibleUrl = ko.observable().extend({ required: true });
            this.name = ko.computed(function () {
                if (_this.url() && _this.database()) {
                    return _this.database() + " on " + _this.url();
                } else if (_this.url()) {
                    return _this.url();
                } else if (_this.database()) {
                    return _this.database();
                }

                return "[empty]";
            });
            this.isValid = ko.computed(function () {
                return _this.url() != null && _this.url().length > 0;
            });
            this.isUserCredentials = ko.observable(false);
            this.isApiKeyCredentials = ko.observable(false);
            this.credentialsType = ko.computed(function () {
                if (_this.isUserCredentials()) {
                    return "user";
                } else if (_this.isApiKeyCredentials()) {
                    return "api-key";
                } else {
                    return "none";
                }
            });
            this.url(dto.Url);
            this.username(dto.Username);
            this.password(dto.Password);
            this.domain(dto.Domain);
            this.apiKey(dto.ApiKey);
            this.database(dto.Database);
            this.transitiveReplicationBehavior(dto.TransitiveReplicationBehavior);
            this.ignoredClient(dto.IgnoredClient);
            this.disabled(dto.Disabled);
            this.clientVisibleUrl(dto.ClientVisibleUrl);

            if (this.username()) {
                this.isUserCredentials(true);
            } else if (this.apiKey()) {
                this.isApiKeyCredentials(true);
            }
        }
        replicationDestination.prototype.toggleUserCredentials = function () {
            this.isUserCredentials.toggle();
            if (this.isUserCredentials()) {
                this.isApiKeyCredentials(false);
            }
        };

        replicationDestination.prototype.toggleApiKeyCredentials = function () {
            this.isApiKeyCredentials.toggle();
            if (this.isApiKeyCredentials()) {
                this.isUserCredentials(false);
            }
        };

        replicationDestination.prototype.toggleIsAdvancedShows = function (item, event) {
            $(event.target).next().toggle();
        };

        replicationDestination.empty = function () {
            return new replicationDestination({
                Url: null,
                Username: null,
                Password: null,
                Domain: null,
                ApiKey: null,
                Database: null,
                TransitiveReplicationBehavior: "None",
                IgnoredClient: false,
                Disabled: false,
                ClientVisibleUrl: null
            });
        };

        replicationDestination.prototype.enable = function () {
            this.disabled(false);
        };

        replicationDestination.prototype.disable = function () {
            this.disabled(true);
        };

        replicationDestination.prototype.includeFailover = function () {
            this.ignoredClient(false);
        };

        replicationDestination.prototype.skipFailover = function () {
            this.ignoredClient(true);
        };

        replicationDestination.prototype.toDto = function () {
            return {
                Url: this.prepareUrl(),
                Username: this.username(),
                Password: this.password(),
                Domain: this.domain(),
                ApiKey: this.apiKey(),
                Database: this.database(),
                TransitiveReplicationBehavior: this.transitiveReplicationBehavior(),
                IgnoredClient: this.ignoredClient(),
                Disabled: this.disabled(),
                ClientVisibleUrl: this.clientVisibleUrl()
            };
        };

        replicationDestination.prototype.prepareUrl = function () {
            var url = this.url();
            if (url && url.charAt(url.length - 1) === "/") {
                url = url.substring(0, url.length - 1);
            }
            return url;
        };
        return replicationDestination;
    })();

    
    return replicationDestination;
});
