var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "plugins/router", "common/appUrl", "common/pagedList", "models/indexPriority", "models/transformer", "models/indexDefinition", "models/customColumns", 'models/customColumnParams', "commands/getConflictsCommand", "commands/getReplicationSourcesCommand", "commands/getIndexDefinitionCommand", "commands/getSingleTransformerCommand", "commands/saveIndexDefinitionCommand", "commands/saveTransformerCommand", "viewmodels/viewModelBase"], function(require, exports, router, appUrl, pagedList, indexPriority, transformer, indexDefinition, customColumns, customColumnParams, getConflictsCommand, getReplicationSourcesCommand, getIndexDefinitionCommand, getSingleTransformerCommand, saveIndexDefinitionCommand, saveTransformerCommand, viewModelBase) {
    var conflicts = (function (_super) {
        __extends(conflicts, _super);
        function conflicts() {
            _super.apply(this, arguments);
            this.displayName = "conflicts";
            this.sourcesLookup = {};
            this.currentColumnsParams = ko.observable(customColumns.empty());
            this.currentConflictsPagedItems = ko.observable();
            this.selectedDocumentIndices = ko.observableArray();
        }
        conflicts.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.activeDatabase.subscribe(function (db) {
                return _this.databaseChanged(db);
            });

            this.currentColumnsParams().columns([
                new customColumnParams({ Header: "Detected At (UTC)", Binding: "conflictDetectedAt", DefaultWidth: 300 }),
                new customColumnParams({ Header: "Versions", Binding: "versions", DefaultWidth: 400, Template: 'versions-template' })
            ]);

            return this.performIndexCheck(this.activeDatabase()).then(function () {
                return _this.loadReplicationSources(_this.activeDatabase());
            }).done(function () {
                _this.fetchConflicts(appUrl.getDatabase());
            });
        };

        conflicts.prototype.fetchConflicts = function (database) {
            this.currentConflictsPagedItems(this.createPagedList(database));
        };

        conflicts.prototype.loadReplicationSources = function (db) {
            var _this = this;
            return new getReplicationSourcesCommand(db).execute().done(function (results) {
                return _this.replicationSourcesLoaded(results, db);
            });
        };

        conflicts.prototype.performIndexCheck = function (db) {
            if (conflicts.performedIndexChecks.contains(db.name)) {
                return $.Deferred().resolve();
            }

            var performCheckTask = $.Deferred();

            $.when(new getIndexDefinitionCommand(conflicts.conflictsIndexName, db).execute(), new getSingleTransformerCommand(conflicts.conflictsTransformerName, db).execute()).done(function () {
                conflicts.performedIndexChecks.push(db.name);
                performCheckTask.resolve();
            }).fail(function () {
                var indexTask = new saveIndexDefinitionCommand(conflicts.getConflictsIndexDefinition(), 0 /* normal */, db).execute();
                var transformerTask = new saveTransformerCommand(conflicts.getConflictsTransformerDefinition(), db).execute();

                $.when(indexTask, transformerTask).done(function () {
                    conflicts.performedIndexChecks.push(db.name);
                    performCheckTask.resolve();
                }).fail(function () {
                    return performCheckTask.reject();
                });
            });

            return performCheckTask;
        };

        conflicts.getConflictsIndexDefinition = function () {
            var indexDef = indexDefinition.empty();
            indexDef.name(conflicts.conflictsIndexName);
            indexDef.maps()[0]("from doc in docs \r\n" + " let id = doc[\"@metadata\"][\"@id\"] \r\n" + " where doc[\"@metadata\"][\"Raven-Replication-Conflict\"] == true && (id.Length < 47 || !id.Substring(id.Length - 47).StartsWith(\"/conflicts/\", StringComparison.OrdinalIgnoreCase)) \r\n" + " select new { ConflictDetectedAt = (DateTime)doc[\"@metadata\"][\"Last-Modified\"] }");

            return indexDef.toDto();
        };
        conflicts.getConflictsTransformerDefinition = function () {
            var transDef = transformer.empty();
            transDef.name(conflicts.conflictsTransformerName);
            transDef.transformResults("from result in results \r\n" + "                select new {  \r\n" + "                    Id = result[\"__document_id\"], \r\n" + "                    ConflictDetectedAt = result[\"@metadata\"].Value<DateTime>(\"Last-Modified\"),  \r\n" + "                    EntityName = result[\"@metadata\"][\"Raven-Entity-Name\"], \r\n" + "                    Versions = result.Conflicts.Select(versionId => { \r\n" + "                        var version = LoadDocument(versionId); \r\n" + "                 return new { Id = versionId, SourceId = version[\"@metadata\"][\"Raven-Replication-Source\"] }; \r\n" + "             }) \r\n" + "         }\r\n");
            return transDef;
        };

        conflicts.prototype.replicationSourcesLoaded = function (sources, db) {
            this.sourcesLookup = sources;
        };

        conflicts.prototype.databaseChanged = function (db) {
            var _this = this;
            var conflictsUrl = appUrl.forConflicts(db);
            router.navigate(conflictsUrl, false);
            this.performIndexCheck(db).then(function () {
                return _this.loadReplicationSources(db);
            }).done(function () {
                _this.fetchConflicts(db);
            });
        };

        conflicts.prototype.createPagedList = function (database) {
            var fetcher = function (skip, take) {
                return new getConflictsCommand(database, skip, take).execute();
            };
            return new pagedList(fetcher);
        };

        conflicts.prototype.getUrlForConflict = function (conflictVersion) {
            return appUrl.forEditDoc(conflictVersion.id, null, 0, this.activeDatabase());
        };

        conflicts.prototype.getTextForVersion = function (conflictVersion) {
            var replicationSource = this.sourcesLookup[conflictVersion.sourceId];
            var text = "";
            if (replicationSource) {
                text = " (" + replicationSource + ")";
            }
            return text;
        };

        conflicts.prototype.getServerUrlForVersion = function (conflictVersion) {
            return this.sourcesLookup[conflictVersion.sourceId] || "";
        };
        conflicts.performedIndexChecks = [];

        conflicts.conflictsIndexName = "Raven/ConflictDocuments";
        conflicts.conflictsTransformerName = "Raven/ConflictDocumentsTransformer";

        conflicts.gridSelector = "#conflictsGrid";
        return conflicts;
    })(viewModelBase);

    
    return conflicts;
});
