var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "durandal/app", "plugins/dialog", "viewmodels/dialogViewModelBase", "commands/commandBase"], function(require, exports, app, dialog, dialogViewModelBase, commandBase) {
    var createDatabase = (function (_super) {
        __extends(createDatabase, _super);
        function createDatabase(databases) {
            _super.call(this);
            this.creationTask = $.Deferred();
            this.creationTaskStarted = false;
            this.databaseName = ko.observable('');
            this.databasePath = ko.observable('');
            this.databaseLogs = ko.observable('');
            this.databaseIndexes = ko.observable('');
            this.databaseNameFocus = ko.observable(true);
            this.isCompressionBundleEnabled = ko.observable(false);
            this.isEncryptionBundleEnabled = ko.observable(false);
            this.isExpirationBundleEnabled = ko.observable(false);
            this.isQuotasBundleEnabled = ko.observable(false);
            this.isReplicationBundleEnabled = ko.observable(false);
            this.isSqlReplicationBundleEnabled = ko.observable(false);
            this.isVersioningBundleEnabled = ko.observable(false);
            this.isPeriodicBackupBundleEnabled = ko.observable(true);
            this.isScriptedIndexBundleEnabled = ko.observable(false);
            this.databases = ko.observableArray();
            this.maxNameLength = 260 - 30;
            this.newCommandBase = new commandBase();
            this.databases = databases;
        }
        createDatabase.prototype.cancel = function () {
            dialog.close(this);
        };

        createDatabase.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.databaseNameFocus(true);
        };

        createDatabase.prototype.deactivate = function () {
            if (!this.creationTaskStarted) {
                this.creationTask.reject();
            }
        };

        createDatabase.prototype.nextOrCreate = function () {
            var databaseName = this.databaseName();
            var databasePath = this.databasePath();
            var databaseLogs = this.databaseLogs();
            var databaseIndexes = this.databaseIndexes();

            if (this.isDatabaseNameLegal(databaseName) && this.arePathsLegal(databasePath, databaseLogs, databaseIndexes)) {
                this.creationTaskStarted = true;
                this.creationTask.resolve(databaseName, this.getActiveBundles(), databasePath, databaseLogs, databaseIndexes);
                dialog.close(this);
            }
        };

        createDatabase.prototype.arePathsLegal = function (databasePath, databaseLogs, databaseIndexes) {
            if (this.isPathLegal(databasePath, "Path") && this.isPathLegal(databaseLogs, "Logs") && this.isPathLegal(databaseIndexes, "Indexes")) {
                return true;
            }
            return false;
        };

        createDatabase.prototype.isPathLegal = function (name, pathName) {
            var rg1 = /^[^\\*:\?"<>\|]+$/;
            var rg2 = /^(nul|prn|con|lpt[0-9]|com[0-9])(\.|$)/i;
            var errorMessage = null;

            if (!$.trim(name) == false) {
                if (name.length > this.maxNameLength) {
                    errorMessage = "The path name for the '" + pathName + "' can't exceed " + this.maxNameLength + " characters!";
                } else if (!rg1.test(name)) {
                    errorMessage = "The " + pathName + " can't contain any of the following characters: * : ?" + ' " ' + "< > |";
                } else if (rg2.test(name)) {
                    errorMessage = "The name '" + name + "' is forbidden for use!";
                }
            }
            if (errorMessage != null) {
                this.newCommandBase.reportError(errorMessage);
                return false;
            }
            return true;
        };

        createDatabase.prototype.isDatabaseNameLegal = function (databaseName) {
            var errorMessage = null;

            if (databaseName == null) {
                errorMessage = "Please fill out the Database Name field";
            } else if (this.isDatabaseNameExists(databaseName, this.databases()) === true) {
                errorMessage = "Database Name Already Exists!";
            } else if ((errorMessage = this.CheckName(databaseName)) != null) {
            }

            if (errorMessage != null) {
                this.newCommandBase.reportError(errorMessage);
                this.databaseNameFocus(true);
                return false;
            }
            return true;
        };

        createDatabase.prototype.CheckName = function (name) {
            var rg1 = /^[^\\/\*:\?"<>\|]+$/;
            var rg2 = /^\./;
            var rg3 = /^(nul|prn|con|lpt[0-9]|com[0-9])(\.|$)/i;

            var message = null;
            if (!$.trim(name)) {
                message = "An empty databse name is forbidden for use!";
            } else if (name.length > this.maxNameLength) {
                message = "The database length can't exceed " + this.maxNameLength + " characters!";
            } else if (!rg1.test(name)) {
                message = "The database name can't contain any of the following characters: \ / * : ?" + ' " ' + "< > |";
            } else if (rg2.test(name)) {
                message = "The database name can't start with a dot!";
            } else if (rg3.test(name)) {
                message = "The name '" + name + "' is forbidden for use!";
            }
            return message;
        };

        createDatabase.prototype.isDatabaseNameExists = function (databaseName, databases) {
            for (var i = 0; i < databases.length; i++) {
                if (databaseName == databases[i].name) {
                    return true;
                }
            }
            return false;
        };

        createDatabase.prototype.toggleCompressionBundle = function () {
            this.isCompressionBundleEnabled.toggle();
        };

        createDatabase.prototype.toggleEncryptionBundle = function () {
            this.isEncryptionBundleEnabled.toggle();
        };

        createDatabase.prototype.toggleExpirationBundle = function () {
            this.isExpirationBundleEnabled.toggle();
        };

        createDatabase.prototype.toggleQuotasBundle = function () {
            if (this.isQuotasBundleEnabled() === false)
                app.showMessage("Quotas Bundle configuration window is not implemented yet.", "Not implemented");
            this.isQuotasBundleEnabled.toggle();
        };

        createDatabase.prototype.toggleReplicationBundle = function () {
            this.isReplicationBundleEnabled.toggle();
        };

        createDatabase.prototype.toggleSqlReplicationBundle = function () {
            this.isSqlReplicationBundleEnabled.toggle();
        };

        createDatabase.prototype.toggleVersioningBundle = function () {
            if (this.isVersioningBundleEnabled() === false)
                app.showMessage("Versioning Bundle configuration window is not implemented yet.", "Not implemented");
            this.isVersioningBundleEnabled.toggle();
        };

        createDatabase.prototype.togglePeriodicBackupBundle = function () {
            this.isPeriodicBackupBundleEnabled.toggle();
        };

        createDatabase.prototype.toggleScriptedIndexBundle = function () {
            this.isScriptedIndexBundleEnabled.toggle();
        };

        createDatabase.prototype.getActiveBundles = function () {
            var activeBundles = [];
            if (this.isCompressionBundleEnabled()) {
                activeBundles.push("Compression");
            }

            if (this.isEncryptionBundleEnabled()) {
                activeBundles.push("Encryption");
            }

            if (this.isExpirationBundleEnabled()) {
                activeBundles.push("DocumentExpiration");
            }

            if (this.isQuotasBundleEnabled()) {
                activeBundles.push("Quotas");
            }

            if (this.isReplicationBundleEnabled()) {
                activeBundles.push("Replication");
            }

            if (this.isSqlReplicationBundleEnabled()) {
                activeBundles.push("SqlReplication");
            }

            if (this.isVersioningBundleEnabled()) {
                activeBundles.push("Versioning");
            }

            if (this.isPeriodicBackupBundleEnabled()) {
                activeBundles.push("PeriodicBackups");
            }

            if (this.isScriptedIndexBundleEnabled()) {
                activeBundles.push("ScriptedIndexResults");
            }
            return activeBundles;
        };
        return createDatabase;
    })(dialogViewModelBase);

    
    return createDatabase;
});
