﻿var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "durandal/app", "plugins/router", "ace/ace", "models/document", "models/collection", "commands/saveDocumentCommand", "viewmodels/deleteDocuments", "common/pagedList", "common/appUrl", "commands/getDocumentWithMetadataCommand", "viewmodels/viewModelBase", "common/alertType", "common/alertArgs", "commands/verifyDocumentsIDsCommand", "common/aceEditorBindingHandler"], function(require, exports, app, router, ace, document, collection, saveDocumentCommand, deleteDocuments, pagedList, appUrl, getDocumentWithMetadataCommand, viewModelBase, alertType, alertArgs, verifyDocumentsIDsCommand, aceEditorBindingHandler) {
    var editDocument = (function (_super) {
        __extends(editDocument, _super);
        function editDocument() {
            var _this = this;
            _super.call(this);
            this.document = ko.observable();
            this.documentText = ko.observable('').extend({ required: true });
            this.metadataText = ko.observable('').extend({ required: true });
            this.isEditingMetadata = ko.observable(false);
            this.isBusy = ko.observable(false);
            this.metaPropsToRestoreOnSave = [];
            this.userSpecifiedId = ko.observable('').extend({ required: true });
            this.isCreatingNewDocument = ko.observable(false);
            this.docsList = ko.observable();
            this.topRecentDocuments = ko.computed(function () {
                return _this.getTopRecentDocuments();
            });
            this.relatedDocumentHrefs = ko.observableArray();
            this.docEditroHasFocus = ko.observable(true);
            this.documentMatchRegexp = /\w+\/\w+/ig;
            this.lodaedDocumentName = ko.observable('');
            aceEditorBindingHandler.install();

            this.metadata = ko.computed(function () {
                return _this.document() ? _this.document().__metadata : null;
            });

            this.document.subscribe(function (doc) {
                if (doc) {
                    var docText = _this.stringify(doc.toDto());
                    _this.documentText(docText);
                }
            });

            this.metadata.subscribe(function (meta) {
                return _this.metadataChanged(meta);
            });
            this.editedDocId = ko.computed(function () {
                return _this.metadata() ? _this.metadata().id : '';
            });
            this.editedDocId.subscribe(function (docId) {
                return ko.postbox.publish("SetRawJSONUrl", appUrl.forDocumentRawData(_this.activeDatabase(), docId));
            });

            this.metadataTextSubscription = this.metadataText.subscribe(function () {
                return _this.updateDocEditorText();
            });
            this.documentTextSubscription = this.documentText.subscribe(function () {
                return _this.updateDocEditorText();
            });
            this.isEditingMetadata.subscribe(function () {
                return _this.updateDocEditorText();
            });
        }
        editDocument.prototype.canActivate = function (args) {
            var _this = this;
            _super.prototype.canActivate.call(this, args);
            if (args && args.id) {
                var canActivateResult = $.Deferred();
                new getDocumentWithMetadataCommand(args.id, appUrl.getDatabase()).execute().done(function (document) {
                    _this.document(document);
                    canActivateResult.resolve({ can: true });

                    var relatedDocumentsCandidates = _this.findRelatedDocumentsCandidates(document);

                    var docIDsVerifyCommand = new verifyDocumentsIDsCommand(relatedDocumentsCandidates, _this.activeDatabase(), true, true);
                    var response = docIDsVerifyCommand.execute();

                    if (response.then) {
                        response.done(function (verifiedIDs) {
                            _this.relatedDocumentHrefs(verifiedIDs.map(function (verified) {
                                return {
                                    id: verified.toString(),
                                    href: appUrl.forEditDoc(verified.toString(), null, null, _this.activeDatabase())
                                };
                            }));
                        });
                    } else {
                        _this.relatedDocumentHrefs(response.map(function (verified) {
                            return {
                                id: verified.toString(),
                                href: appUrl.forEditDoc(verified.toString(), null, null, _this.activeDatabase())
                            };
                        }));
                    }
                }).fail(function () {
                    ko.postbox.publish("Alert", new alertArgs(3 /* danger */, "Could not find " + args.id + " document", null));
                    canActivateResult.resolve({ redirect: appUrl.forDocuments(collection.allDocsCollectionName, _this.activeDatabase()) });
                });
                return canActivateResult;
            } else {
                return $.Deferred().resolve({ can: true });
            }
        };

        editDocument.prototype.activate = function (navigationArgs) {
            var _this = this;
            _super.prototype.activate.call(this, navigationArgs);

            this.lodaedDocumentName(this.userSpecifiedId());
            viewModelBase.dirtyFlag = new ko.DirtyFlag([this.documentText, this.metadataText, this.userSpecifiedId]);
            var self = this;
            this.isSaveEnabled = ko.computed(function () {
                return viewModelBase.dirtyFlag().isDirty() && !!self.userSpecifiedId();
            });

            this.databaseForEditedDoc = this.activeDatabase();
            if (navigationArgs && navigationArgs.database) {
                ko.postbox.publish("ActivateDatabaseWithName", navigationArgs.database);
            }

            if (navigationArgs && navigationArgs.list && navigationArgs.item) {
                var itemIndex = parseInt(navigationArgs.item, 10);
                if (!isNaN(itemIndex)) {
                    var newCollection = new collection(navigationArgs.list, appUrl.getDatabase());
                    var fetcher = function (skip, take) {
                        return newCollection.fetchDocuments(skip, take);
                    };
                    var list = new pagedList(fetcher);
                    list.collectionName = navigationArgs.list;
                    list.currentItemIndex(itemIndex);
                    list.getNthItem(0);
                    this.docsList(list);
                }
            }

            if (navigationArgs && navigationArgs.id) {
                var existingRecentDocumentsStore = editDocument.recentDocumentsInDatabases.first(function (x) {
                    return x.databaseName == _this.databaseForEditedDoc.name;
                });
                if (existingRecentDocumentsStore) {
                    var existingDocumentInStore = existingRecentDocumentsStore.recentDocuments.first(function (x) {
                        return x === navigationArgs.id;
                    });
                    if (!existingDocumentInStore) {
                        if (existingRecentDocumentsStore.recentDocuments().length == 5) {
                            existingRecentDocumentsStore.recentDocuments.pop();
                        }
                        existingRecentDocumentsStore.recentDocuments.unshift(navigationArgs.id);
                    }
                } else {
                    editDocument.recentDocumentsInDatabases.push({ databaseName: this.databaseForEditedDoc.name, recentDocuments: ko.observableArray([navigationArgs.id]) });
                }

                ko.postbox.publish("SetRawJSONUrl", appUrl.forDocumentRawData(this.activeDatabase(), navigationArgs.id));
                return true;
            } else {
                this.editNewDocument();
            }
        };

        editDocument.prototype.attached = function () {
            this.initializeDocEditor();
            this.setupKeyboardShortcuts();
            this.focusOnEditor();
        };

        editDocument.prototype.initializeDocEditor = function () {
            var _this = this;
            this.docEditor = ace.edit("docEditor");
            this.docEditor.setTheme("ace/theme/github");
            this.docEditor.setFontSize("16px");
            this.docEditor.getSession().setMode("ace/mode/json");
            $("#docEditor").on('keyup', ".ace_text-input", function () {
                return _this.storeDocEditorTextIntoObservable();
            });
            this.updateDocEditorText();
        };

        editDocument.prototype.setupKeyboardShortcuts = function () {
            var _this = this;
            this.createKeyboardShortcut("alt+shift+d", function () {
                return _this.focusOnDocument();
            }, editDocument.editDocSelector);
            this.createKeyboardShortcut("alt+shift+m", function () {
                return _this.focusOnMetadata();
            }, editDocument.editDocSelector);
            this.createKeyboardShortcut("alt+c", function () {
                return _this.focusOnEditor();
            }, editDocument.editDocSelector);
            this.createKeyboardShortcut("alt+home", function () {
                return _this.firstDocument();
            }, editDocument.editDocSelector);
            this.createKeyboardShortcut("alt+end", function () {
                return _this.lastDocument();
            }, editDocument.editDocSelector);
            this.createKeyboardShortcut("alt+page-up", function () {
                return _this.previousDocumentOrLast();
            }, editDocument.editDocSelector);
            this.createKeyboardShortcut("alt+page-down", function () {
                return _this.nextDocumentOrFirst();
            }, editDocument.editDocSelector);
            this.createKeyboardShortcut("alt+shift+del", function () {
                return _this.deleteDocument();
            }, editDocument.editDocSelector);
        };

        editDocument.prototype.focusOnMetadata = function () {
            this.isEditingMetadata(true);
            this.focusOnEditor();
        };

        editDocument.prototype.focusOnDocument = function () {
            this.isEditingMetadata(false);
            this.focusOnEditor();
        };

        editDocument.prototype.focusOnEditor = function () {
            this.docEditor.focus();
        };

        editDocument.prototype.editNewDocument = function () {
            this.isCreatingNewDocument(true);
            this.document(document.empty());
        };

        editDocument.prototype.failedToLoadDoc = function (docId, errorResponse) {
            ko.postbox.publish("Alert", new alertArgs(3 /* danger */, "Could not find " + docId + " document", null));
        };

        editDocument.prototype.saveDocument = function () {
            var _this = this;
            var meta = JSON.parse(this.metadataText());
            var currentDocumentId = this.userSpecifiedId();
            if (!!this.lodaedDocumentName() && this.lodaedDocumentName() != currentDocumentId) {
                this.isCreatingNewDocument(true);
            }

            var updatedDto = JSON.parse(this.documentText());
            updatedDto['@metadata'] = meta;

            if (this.isCreatingNewDocument()) {
                this.attachReservedMetaProperties(currentDocumentId, meta);
            } else {
                this.metaPropsToRestoreOnSave.forEach(function (p) {
                    if (p.name !== "Origin") {
                        meta[p.name] = p.value;
                    }
                });
            }

            var newDoc = new document(updatedDto);
            var saveCommand = new saveDocumentCommand(currentDocumentId, newDoc, appUrl.getDatabase());
            var saveTask = saveCommand.execute();
            saveTask.done(function (idAndEtag) {
                viewModelBase.dirtyFlag().reset();

                _this.lodaedDocumentName(currentDocumentId);
                _this.isCreatingNewDocument(false);
                _this.loadDocument(idAndEtag.Key);
                _this.updateUrl(idAndEtag.Key);
            });
        };

        editDocument.prototype.attachReservedMetaProperties = function (id, target) {
            target['@etag'] = '';
            target['Raven-Entity-Name'] = document.getEntityNameFromId(id);
            target['@id'] = id;
        };

        editDocument.prototype.stringify = function (obj) {
            var prettifySpacing = 4;
            return JSON.stringify(obj, null, prettifySpacing);
        };

        editDocument.prototype.activateMeta = function () {
            this.isEditingMetadata(true);
        };

        editDocument.prototype.activateDoc = function () {
            this.isEditingMetadata(false);
        };

        editDocument.prototype.findRelatedDocumentsCandidates = function (doc) {
            var results = [];
            var initialDocumentFields = doc.getDocumentPropertyNames();
            var documentNodesFlattenedList = [];

            initialDocumentFields.forEach(function (curField) {
                documentNodesFlattenedList.push(doc[curField]);
            });

            for (var documentNodesCursor = 0; documentNodesCursor < documentNodesFlattenedList.length; documentNodesCursor++) {
                var curField = documentNodesFlattenedList[documentNodesCursor];
                if (typeof curField === "string" && /\w+\/\w+/ig.test(curField)) {
                    if (!results.first(function (x) {
                        return x === curField.toString();
                    })) {
                        results.push(curField.toString());
                    }
                } else if (typeof curField == "object" && !!curField) {
                    for (var curInnerField in curField) {
                        documentNodesFlattenedList.push(curField[curInnerField]);
                    }
                }
            }
            return results;
        };

        editDocument.prototype.loadDocument = function (id) {
            var _this = this;
            var loadDocTask = new getDocumentWithMetadataCommand(id, this.databaseForEditedDoc).execute();
            loadDocTask.done(function (document) {
                _this.document(document);

                viewModelBase.dirtyFlag().reset();
            });
            loadDocTask.fail(function (response) {
                return _this.failedToLoadDoc(id, response);
            });
            loadDocTask.always(function () {
                return _this.isBusy(false);
            });
            this.isBusy(true);
            return loadDocTask;
        };

        editDocument.prototype.refreshDocument = function () {
            var meta = this.metadata();
            if (!this.isCreatingNewDocument()) {
                var docId = this.editedDocId();
                this.document(null);
                this.documentText(null);
                this.metadataText(null);
                this.userSpecifiedId('');
                this.loadDocument(docId);
            } else {
                this.editNewDocument();
            }
        };

        editDocument.prototype.deleteDocument = function () {
            var _this = this;
            var doc = this.document();
            if (doc) {
                var viewModel = new deleteDocuments([doc]);
                viewModel.deletionTask.done(function () {
                    return _this.nextDocumentOrFirst();
                });
                app.showDialog(viewModel, editDocument.editDocSelector);
            }

            viewModelBase.dirtyFlag().reset();
        };

        editDocument.prototype.formatDocument = function () {
            var docText = this.documentText();
            var tempDoc = JSON.parse(docText);
            var formatted = this.stringify(tempDoc);
            this.documentText(formatted);
        };

        editDocument.prototype.nextDocumentOrFirst = function () {
            var list = this.docsList();
            if (list) {
                var nextIndex = list.currentItemIndex() + 1;
                if (nextIndex >= list.totalResultCount()) {
                    nextIndex = 0;
                }
                this.pageToItem(nextIndex);
            } else {
                this.navigateToDocuments();
            }
        };

        editDocument.prototype.previousDocumentOrLast = function () {
            var list = this.docsList();
            if (list) {
                var previousIndex = list.currentItemIndex() - 1;
                if (previousIndex < 0) {
                    previousIndex = list.totalResultCount() - 1;
                }
                this.pageToItem(previousIndex);
            }
        };

        editDocument.prototype.lastDocument = function () {
            var list = this.docsList();
            if (list) {
                this.pageToItem(list.totalResultCount() - 1);
            }
        };

        editDocument.prototype.firstDocument = function () {
            this.pageToItem(0);
        };

        editDocument.prototype.pageToItem = function (index) {
            var _this = this;
            var list = this.docsList();
            if (list) {
                list.getNthItem(index).done(function (doc) {
                    _this.loadDocument(doc.getId());
                    list.currentItemIndex(index);
                    _this.updateUrl(doc.getId());
                });
            }
        };

        editDocument.prototype.navigateToCollection = function (collectionName) {
            var collectionUrl = appUrl.forDocuments(collectionName, this.activeDatabase());
            router.navigate(collectionUrl);
        };

        editDocument.prototype.navigateToDocuments = function () {
            this.navigateToCollection(null);
        };

        editDocument.prototype.updateUrl = function (docId) {
            var collectionName = this.docsList() ? this.docsList().collectionName : null;
            var currentItemIndex = this.docsList() ? this.docsList().currentItemIndex() : null;
            var editDocUrl = appUrl.forEditDoc(docId, collectionName, currentItemIndex, this.activeDatabase());
            router.navigate(editDocUrl, false);
        };

        editDocument.prototype.updateDocEditorText = function () {
            if (this.docEditor) {
                var text = this.isEditingMetadata() ? this.metadataText() : this.documentText();
                this.docEditor.getSession().setValue(text);
            }
        };

        editDocument.prototype.storeDocEditorTextIntoObservable = function () {
            var _this = this;
            if (this.docEditor) {
                var docEditorText = this.docEditor.getSession().getValue();
                var observableToUpdate = this.isEditingMetadata() ? this.metadataText : this.documentText;
                var subscription = this.isEditingMetadata() ? this.metadataTextSubscription : this.documentTextSubscription;

                subscription.dispose();
                observableToUpdate(docEditorText);
                if (this.isEditingMetadata()) {
                    this.metadataTextSubscription = this.metadataText.subscribe(function () {
                        return _this.updateDocEditorText();
                    });
                } else {
                    this.documentTextSubscription = this.documentText.subscribe(function () {
                        return _this.updateDocEditorText();
                    });
                }
            }
        };

        editDocument.prototype.getTopRecentDocuments = function () {
            var _this = this;
            var currentDbName = this.activeDatabase().name;
            var recentDocumentsForCurDb = editDocument.recentDocumentsInDatabases().first(function (x) {
                return x.databaseName === currentDbName;
            });
            if (recentDocumentsForCurDb) {
                var value = recentDocumentsForCurDb.recentDocuments().slice(0, 5).map(function (docId) {
                    return {
                        docId: docId,
                        docUrl: appUrl.forEditDoc(docId, null, null, _this.activeDatabase())
                    };
                });
                return value;
            } else {
                return [];
            }
        };

        editDocument.prototype.metadataChanged = function (meta) {
            if (meta) {
                this.metaPropsToRestoreOnSave.length = 0;
                var metaDto = this.metadata().toDto();

                var metaPropsToRemove = [
                    "@id", "@etag", "Origin", "Raven-Server-Build", "Raven-Client-Version", "Non-Authoritative-Information", "Raven-Timer-Request",
                    "Raven-Authenticated-User", "Raven-Last-Modified", "Has-Api-Key", "Access-Control-Allow-Origin", "Access-Control-Max-Age", "Access-Control-Allow-Methods",
                    "Access-Control-Request-Headers", "Access-Control-Allow-Headers", "Reverse-Via", "Persistent-Auth", "Allow", "Content-Disposition", "Content-Encoding",
                    "Content-Language", "Content-Location", "Content-MD5", "Content-Range", "Content-Type", "Expires", "Last-Modified", "Content-Length", "Keep-Alive", "X-Powered-By",
                    "X-AspNet-Version", "X-Requested-With", "X-SourceFiles", "Accept-Charset", "Accept-Encoding", "Accept", "Accept-Language", "Authorization", "Cookie", "Expect",
                    "From", "Host", "If-MatTemp-Index-Scorech", "If-Modified-Since", "If-None-Match", "If-Range", "If-Unmodified-Since", "Max-Forwards", "Referer", "TE", "User-Agent", "Accept-Ranges",
                    "Age", "Allow", "ETag", "Location", "Retry-After", "Server", "Set-Cookie2", "Set-Cookie", "Vary", "Www-Authenticate", "Cache-Control", "Connection", "Date", "Pragma",
                    "Trailer", "Transfer-Encoding", "Upgrade", "Via", "Warning", "X-ARR-LOG-ID", "X-ARR-SSL", "X-Forwarded-For", "X-Original-URL"];

                for (var property in metaDto) {
                    if (metaDto.hasOwnProperty(property) && metaPropsToRemove.contains(property)) {
                        if (metaDto[property]) {
                            this.metaPropsToRestoreOnSave.push({ name: property, value: metaDto[property].toString() });
                        }
                        delete metaDto[property];
                    }
                }

                var metaString = this.stringify(metaDto);
                this.metadataText(metaString);
                this.userSpecifiedId(meta.id);
            }
        };
        editDocument.editDocSelector = "#editDocumentContainer";
        editDocument.recentDocumentsInDatabases = ko.observableArray();
        return editDocument;
    })(viewModelBase);

    
    return editDocument;
});
