﻿var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "durandal/app", "plugins/router", "common/appUrl", "ace/ace", "commands/filesystem/getFileCommand", "commands/filesystem/updateFileMetadataCommand", "viewmodels/viewModelBase", "viewmodels/deleteItems"], function(require, exports, app, router, appUrl, ace, getFileCommand, updateFileMetadataCommand, viewModelBase, deleteItems) {
    var filesystemEditFile = (function (_super) {
        __extends(filesystemEditFile, _super);
        function filesystemEditFile() {
            var _this = this;
            _super.call(this);
            this.fileName = ko.observable();
            this.file = ko.observable();
            this.fileMetadataText = ko.observable();
            this.isBusy = ko.observable(false);
            this.metaPropsToRestoreOnSave = [];

            this.fileMetadataText.subscribe(function () {
                return _this.updateFileEditorText();
            });
            this.fileName.subscribe(function (x) {
                return _this.loadFile(x);
            });
        }
        filesystemEditFile.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.metadata = ko.computed(function () {
                return _this.file() ? _this.file().__metadata : null;
            });

            if (args.id != null) {
                this.fileName(args.id);
            }

            this.metadata.subscribe(function (meta) {
                return _this.metadataChanged(meta);
            });
        };

        filesystemEditFile.prototype.attached = function () {
            this.initializeFileEditor();

            this.focusOnEditor();
        };

        filesystemEditFile.prototype.initializeFileEditor = function () {
            var _this = this;
            this.fileMetadataEditor = ace.edit("fileMetadataEditor");
            this.fileMetadataEditor.setTheme("ace/theme/github");
            this.fileMetadataEditor.setFontSize("16px");
            this.fileMetadataEditor.getSession().setMode("ace/mode/json");
            $("#fileMetadataEditor").on('blur', ".ace_text-input", function () {
                return _this.storeFileEditorTextIntoObservable();
            });
            this.updateFileEditorText();
        };

        filesystemEditFile.prototype.focusOnEditor = function () {
            this.fileMetadataEditor.focus();
        };

        filesystemEditFile.prototype.updateFileEditorText = function () {
            if (this.fileMetadataEditor) {
                this.fileMetadataEditor.getSession().setValue(this.fileMetadataText());
            }
        };

        filesystemEditFile.prototype.storeFileEditorTextIntoObservable = function () {
            if (this.fileMetadataEditor) {
                var editorText = this.fileMetadataEditor.getSession().getValue();
                this.fileMetadataText(editorText);
            }
        };

        filesystemEditFile.prototype.loadFile = function (fileName) {
            var _this = this;
            new getFileCommand(this.activeFilesystem(), fileName).execute().done(function (result) {
                return _this.file(result);
            });
        };

        filesystemEditFile.prototype.navigateToFiles = function () {
            var filesUrl = appUrl.forFilesystemFiles(this.activeFilesystem());
            router.navigate(filesUrl);
        };

        filesystemEditFile.prototype.saveFileMetadata = function () {
            var _this = this;
            var meta = JSON.parse(this.fileMetadataText());
            var currentDocumentId = this.fileName();

            this.metaPropsToRestoreOnSave.forEach(function (p) {
                return meta[p.name] = p.value;
            });

            var saveCommand = new updateFileMetadataCommand(this.fileName(), meta, this.activeFilesystem(), true);
            var saveTask = saveCommand.execute();
            saveTask.done(function () {
                viewModelBase.dirtyFlag().reset();

                _this.loadFile(_this.fileName());
            });
        };

        filesystemEditFile.prototype.downloadFile = function () {
            var url = appUrl.forResourceQuery(this.activeFilesystem()) + "/files/" + this.fileName();
            window.location.assign(url);
        };

        filesystemEditFile.prototype.refreshFile = function () {
            this.loadFile(this.fileName());
        };

        filesystemEditFile.prototype.saveInObservable = function () {
            this.storeFileEditorTextIntoObservable();
        };

        filesystemEditFile.prototype.deleteFile = function () {
            var _this = this;
            var file = this.file();
            if (file) {
                var viewModel = new deleteItems([file]);
                viewModel.deletionTask.done(function () {
                    var filesUrl = appUrl.forFilesystemFiles(_this.activeFilesystem());
                    router.navigate(filesUrl);
                });
                app.showDialog(viewModel, filesystemEditFile.editFileSelector);
            }

            viewModelBase.dirtyFlag().reset();
        };

        filesystemEditFile.prototype.metadataChanged = function (meta) {
            if (meta) {
                var metaDto = this.metadata().toDto();

                var metaPropsToRemove = [
                    "Origin", "Raven-Server-Build", "Raven-Client-Version", "Non-Authoritative-Information", "Raven-Timer-Request",
                    "Raven-Authenticated-User", "Raven-Last-Modified", "Has-Api-Key", "Access-Control-Allow-Origin", "Access-Control-Max-Age", "Access-Control-Allow-Methods",
                    "Access-Control-Request-Headers", "Access-Control-Allow-Headers", "Reverse-Via", "Persistent-Auth", "Allow", "Content-Disposition", "Content-Encoding",
                    "Content-Language", "Content-Location", "Content-MD5", "Content-Range", "Content-Type", "Expires", "Last-Modified", "Content-Length", "Keep-Alive", "X-Powered-By",
                    "X-AspNet-Version", "X-Requested-With", "X-SourceFiles", "Accept-Charset", "Accept-Encoding", "Accept", "Accept-Language", "Authorization", "Cookie", "Expect",
                    "From", "Host", "If-MatTemp-Index-Scorech", "If-Modified-Since", "If-None-Match", "If-Range", "If-Unmodified-Since", "Max-Forwards", "Referer", "TE", "User-Agent", "Accept-Ranges",
                    "Age", "Allow", "ETag", "Location", "Retry-After", "Server", "Set-Cookie2", "Set-Cookie", "Vary", "Www-Authenticate", "Cache-Control", "Connection", "Date", "Pragma",
                    "Trailer", "Transfer-Encoding", "Upgrade", "Via", "Warning", "X-ARR-LOG-ID", "X-ARR-SSL", "X-Forwarded-For", "X-Original-URL",
                    "RavenFS-Size", "Temp-Request-Time", "DNT"];

                for (var property in metaDto) {
                    if (metaDto.hasOwnProperty(property) && metaPropsToRemove.contains(property)) {
                        if (metaDto[property]) {
                            this.metaPropsToRestoreOnSave.push({ name: property, value: metaDto[property].toString() });
                        }
                        delete metaDto[property];
                    }
                }

                var metaString = this.stringify(metaDto);
                this.fileMetadataText(metaString);
            }
        };

        filesystemEditFile.prototype.stringify = function (obj) {
            var prettifySpacing = 4;
            return JSON.stringify(obj, null, prettifySpacing);
        };
        filesystemEditFile.editFileSelector = "#editFileContainer";
        return filesystemEditFile;
    })(viewModelBase);

    
    return filesystemEditFile;
});
