﻿var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "durandal/app", "plugins/router", "common/appUrl", "viewmodels/viewModelBase", "models/filesystem/synchronizationDestination", "commands/filesystem/getDestinationsCommand", "commands/filesystem/getFilesConflictsCommand", "commands/filesystem/getSyncOutgoingActivitiesCommand", "commands/filesystem/getSyncIncomingActivitiesCommand", "commands/filesystem/saveDestinationCommand", "commands/filesystem/deleteDestinationCommand", "commands/filesystem/synchronizeNowCommand", "commands/filesystem/synchronizeWithDestinationCommand"], function(require, exports, app, router, appUrl, viewModelBase, synchronizationDestination, getDestinationsCommand, getFilesConflictsCommand, getSyncOutgoingActivitiesCommand, getSyncIncomingActivitiesCommand, saveDestinationCommand, deleteDestinationCommand, synchronizeNowCommand, synchronizeWithDestinationCommand) {
    var filesystemSynchronization = (function (_super) {
        __extends(filesystemSynchronization, _super);
        function filesystemSynchronization() {
            var _this = this;
            _super.call(this);
            this.destinations = ko.observableArray();
            this.isDestinationsVisible = ko.computed(function () {
                return _this.destinations().length > 0;
            });
            this.conflicts = ko.observableArray();
            this.isConflictsVisible = ko.computed(function () {
                return _this.conflicts().length > 0;
            });
            this.outgoingActivity = ko.observableArray();
            this.isOutgoingActivityVisible = ko.computed(function () {
                return _this.outgoingActivity().length > 0;
            });
            this.incomingActivity = ko.observableArray();
            this.isIncomingActivityVisible = ko.computed(function () {
                return _this.incomingActivity().length > 0;
            });
            this.router = router;
            this.synchronizationUrl = appUrl.forCurrentDatabase().filesystemSynchronization;
        }
        filesystemSynchronization.prototype.canActivate = function (args) {
            return true;
        };

        filesystemSynchronization.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
        };

        filesystemSynchronization.prototype.addDestination = function () {
            var _this = this;
            var fs = this.activeFilesystem();
            require(["viewmodels/filesystem/filesystemAddDestination"], function (filesystemAddDestination) {
                var addDestinationViewModel = new filesystemAddDestination(_this.destinations);
                addDestinationViewModel.creationTask.done(function (destinationUrl) {
                    return _this.addDestinationUrl(new synchronizationDestination(fs, destinationUrl));
                });
                app.showDialog(addDestinationViewModel);
            });
        };

        filesystemSynchronization.prototype.addDestinationUrl = function (url) {
            var fs = this.activeFilesystem();
            if (fs) {
                var self = this;
                new saveDestinationCommand(fs, url).execute().done(function (x) {
                    return self.forceModelPolling();
                });
            }
        };

        filesystemSynchronization.prototype.synchronizeNow = function () {
            var fs = this.activeFilesystem();
            if (fs) {
                new synchronizeNowCommand(fs).execute();
            }
        };

        filesystemSynchronization.prototype.synchronizeWithDestination = function (destination) {
            var fs = this.activeFilesystem();
            if (fs) {
                new synchronizeWithDestinationCommand(fs, destination).execute();
            }
        };

        filesystemSynchronization.prototype.deleteDestination = function (destination) {
            var fs = this.activeFilesystem();
            var self = this;
            if (fs) {
                new deleteDestinationCommand(fs, destination).execute().done(function (x) {
                    return self.forceModelPolling();
                });
            }
        };

        filesystemSynchronization.prototype.modelPolling = function () {
            var _this = this;
            var fs = this.activeFilesystem();
            if (fs) {
                new getDestinationsCommand(fs).execute().done(function (data) {
                    return _this.destinations(data);
                });

                new getFilesConflictsCommand(fs).execute().done(function (x) {
                    return _this.conflicts(x);
                });

                new getSyncOutgoingActivitiesCommand(fs).execute().done(function (x) {
                    return _this.outgoingActivity(x);
                });

                new getSyncIncomingActivitiesCommand(fs).execute().done(function (x) {
                    return _this.incomingActivity(x);
                });
            }
        };

        filesystemSynchronization.prototype.collapseAll = function () {
            $(".synchronization-group-content").collapse('hide');
        };

        filesystemSynchronization.prototype.expandAll = function () {
            $(".synchronization-group-content").collapse('show');
        };
        return filesystemSynchronization;
    })(viewModelBase);

    
    return filesystemSynchronization;
});
