﻿var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "durandal/app", "plugins/router", "common/appUrl", "viewmodels/viewModelBase", "commands/filesystem/searchByQueryCommand", "common/pagedList"], function(require, exports, app, router, appUrl, viewModelBase, searchByQueryCommand, pagedList) {
    var search = (function (_super) {
        __extends(search, _super);
        function search() {
            var _this = this;
            _super.call(this);
            this.router = router;
            this.searchUrl = appUrl.forCurrentDatabase().filesystemSearch;
            this.searchText = ko.observable("");
            this.allFilesPagedItems = ko.observable();
            this.selectedFilesIndices = ko.observableArray();

            this.searchText.extend({ throttle: 200 }).subscribe(function (s) {
                return _this.searchFiles(s);
            });
        }
        search.prototype.canActivate = function (args) {
            return true;
        };

        search.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);

            this.loadFiles(false);
        };

        search.prototype.attached = function () {
        };

        search.prototype.searchFiles = function (query) {
            var _this = this;
            var fetcher = function (skip, take) {
                return _this.fetchFiles(query, skip, take);
            };
            var list = new pagedList(fetcher);
            this.allFilesPagedItems(list);
        };

        search.prototype.loadFiles = function (force) {
            if (!this.allFilesPagedItems() || force) {
                this.allFilesPagedItems(this.createPagedList());
            }

            return this.allFilesPagedItems;
        };

        search.prototype.createPagedList = function () {
            var _this = this;
            var fetcher = function (skip, take) {
                return _this.fetchFiles("", skip, take);
            };
            var list = new pagedList(fetcher);
            return list;
        };

        search.prototype.fetchFiles = function (query, skip, take) {
            var task = new searchByQueryCommand(appUrl.getFilesystem(), query, skip, take, null).execute();
            return task;
        };

        search.prototype.fileNameStartsWith = function () {
            var _this = this;
            require(["viewmodels/filesystem/searchSingleInputClause"], function (searchSingleInputClause) {
                var searchSingleInputClauseViewModel = new searchSingleInputClause("Filename starts with: ");
                searchSingleInputClauseViewModel.applyFilterTask.done(function (input) {
                    return _this.searchText("__fileName:" + input + "*");
                });
                app.showDialog(searchSingleInputClauseViewModel);
            });
        };

        search.prototype.fileNameEndsWith = function () {
            var _this = this;
            require(["viewmodels/filesystem/searchSingleInputClause"], function (searchSingleInputClause) {
                var searchSingleInputClauseViewModel = new searchSingleInputClause("Filename ends with: ");
                searchSingleInputClauseViewModel.applyFilterTask.done(function (input) {
                    return _this.searchText("__rfileName:" + String.prototype.reverse(input) + "*");
                });
                app.showDialog(searchSingleInputClauseViewModel);
            });
        };

        search.prototype.fileSizeBetween = function () {
            var _this = this;
            require(["viewmodels/filesystem/searchFileSizeRangeClause"], function (searchFileSizeRangeClause) {
                var searchFileSizeRangeClauseViewModel = new searchFileSizeRangeClause();
                searchFileSizeRangeClauseViewModel.applyFilterTask.done(function (input) {
                    return _this.searchText(input);
                });
                app.showDialog(searchFileSizeRangeClauseViewModel);
            });
        };

        search.prototype.hasMetadata = function () {
            var _this = this;
            require(["viewmodels/filesystem/searchHasMetadataClause"], function (searchHasMetadataClause) {
                var searchHasMetadataClauseViewModel = new searchHasMetadataClause();
                searchHasMetadataClauseViewModel.applyFilterTask.done(function (input) {
                    return _this.searchText(input);
                });
                app.showDialog(searchHasMetadataClauseViewModel);
            });
        };
        search.gridSelector = "#filesGrid";
        return search;
    })(viewModelBase);

    
    return search;
});
