﻿var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "commands/getDatabaseStatsCommand", "viewmodels/viewModelBase", "models/index", "common/appUrl", "commands/saveIndexLockModeCommand", "commands/saveIndexAsPersistentCommand", "viewmodels/deleteIndexesConfirm", "commands/getStoredQueriesCommand", "models/querySort", "durandal/app", "viewmodels/resetIndexConfirm"], function(require, exports, getDatabaseStatsCommand, viewModelBase, index, appUrl, saveIndexLockModeCommand, saveIndexAsPersistentCommand, deleteIndexesConfirm, getStoredQueriesCommand, querySort, app, resetIndexConfirm) {
    var indexes = (function (_super) {
        __extends(indexes, _super);
        function indexes() {
            _super.apply(this, arguments);
            var _this = this;
            this.indexGroups = ko.observableArray();
            this.queryUrl = ko.observable();
            this.newIndexUrl = appUrl.forCurrentDatabase().newIndex;
            this.containerSelector = "#indexesContainer";
            this.recentQueries = ko.observableArray();
            this.sortedGroups = ko.computed(function () {
                var groups = _this.indexGroups().slice(0).sort(function (l, r) {
                    return l.entityName.toLowerCase() > r.entityName.toLowerCase() ? 1 : -1;
                });

                groups.forEach(function (group) {
                    group.indexes(group.indexes().slice(0).sort(function (l, r) {
                        return l.name.toLowerCase() > r.name.toLowerCase() ? 1 : -1;
                    }));
                });

                return groups;
            });
        }
        indexes.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);

            this.queryUrl(appUrl.forQuery(this.activeDatabase(), null));

            this.fetchIndexes();
            this.fetchRecentQueries();
        };

        indexes.prototype.modelPolling = function () {
        };

        indexes.prototype.attached = function () {
            var _this = this;
            this.createKeyboardShortcut("Alt+=", function () {
                return _this.expandAll();
            }, this.containerSelector);
            ko.postbox.publish("SetRawJSONUrl", appUrl.forIndexesRawData(this.activeDatabase()));
        };

        indexes.prototype.fetchIndexes = function () {
            var _this = this;
            new getDatabaseStatsCommand(this.activeDatabase()).execute().done(function (stats) {
                return _this.processDbStats(stats);
            });
        };

        indexes.prototype.fetchRecentQueries = function () {
            var _this = this;
            new getStoredQueriesCommand(this.activeDatabase()).execute().done(function (doc) {
                return _this.recentQueries(doc.Queries);
            });
        };

        indexes.prototype.getRecentQueryUrl = function (query) {
            return appUrl.forQuery(this.activeDatabase(), query.Hash);
        };

        indexes.prototype.getRecentQuerySortText = function (query) {
            if (query.Sorts.length === 0) {
                return "";
            }
            return query.Sorts.map(function (s) {
                return querySort.fromQuerySortString(s);
            }).map(function (s) {
                return s.toHumanizedString();
            }).reduce(function (first, second) {
                return first + ", " + second;
            });
        };

        indexes.prototype.processDbStats = function (stats) {
            var _this = this;
            stats.Indexes.map(function (i) {
                return new index(i);
            }).forEach(function (i) {
                return _this.putIndexIntoGroups(i);
            });
        };

        indexes.prototype.putIndexIntoGroups = function (i) {
            var _this = this;
            if (i.forEntityName.length === 0) {
                this.putIndexIntoGroupNamed(i, "Other");
            } else {
                i.forEntityName.forEach(function (e) {
                    return _this.putIndexIntoGroupNamed(i, e);
                });
            }
        };

        indexes.prototype.putIndexIntoGroupNamed = function (i, groupName) {
            var group = this.indexGroups.first(function (g) {
                return g.entityName === groupName;
            });
            var indexExists;
            if (group) {
                indexExists = !!group.indexes.first(function (cur) {
                    return cur.name == i.name;
                });
                if (!indexExists) {
                    group.indexes.push(i);
                }
            } else {
                indexExists = !!this.indexGroups.first(function (curGroup) {
                    return !!curGroup.indexes.first(function (cur) {
                        return cur.name == i.name;
                    });
                });

                if (!indexExists) {
                    this.indexGroups.push({ entityName: groupName, indexes: ko.observableArray([i]) });
                }
            }
        };

        indexes.prototype.collapseAll = function () {
            $(".index-group-content").collapse('hide');
        };

        indexes.prototype.expandAll = function () {
            $(".index-group-content").collapse('show');
        };

        indexes.prototype.deleteIdleIndexes = function () {
            var idleIndexes = this.getAllIndexes().filter(function (i) {
                return i.priority && i.priority.indexOf("Idle") !== -1;
            });
            this.promptDeleteIndexes(idleIndexes);
        };

        indexes.prototype.deleteDisabledIndexes = function () {
            var abandonedIndexes = this.getAllIndexes().filter(function (i) {
                return i.priority && i.priority.indexOf("Disabled") !== -1;
            });
            this.promptDeleteIndexes(abandonedIndexes);
        };

        indexes.prototype.deleteAbandonedIndexes = function () {
            var abandonedIndexes = this.getAllIndexes().filter(function (i) {
                return i.priority && i.priority.indexOf("Abandoned") !== -1;
            });
            this.promptDeleteIndexes(abandonedIndexes);
        };

        indexes.prototype.deleteAllIndexes = function () {
            this.promptDeleteIndexes(this.getAllIndexes());
        };

        indexes.prototype.deleteIndex = function (i) {
            this.promptDeleteIndexes([i]);
        };

        indexes.prototype.deleteIndexGroup = function (i) {
            this.promptDeleteIndexes(i.indexes());
        };

        indexes.prototype.promptDeleteIndexes = function (indexes) {
            var _this = this;
            if (indexes.length > 0) {
                var deleteIndexesVm = new deleteIndexesConfirm(indexes.map(function (i) {
                    return i.name;
                }), this.activeDatabase());
                app.showDialog(deleteIndexesVm);
                deleteIndexesVm.deleteTask.done(function () {
                    return _this.removeIndexesFromAllGroups(indexes);
                });
            }
        };

        indexes.prototype.resetIndex = function (indexToReset) {
            var resetIndexVm = new resetIndexConfirm(indexToReset.name, this.activeDatabase());
            app.showDialog(resetIndexVm);
        };

        indexes.prototype.removeIndexesFromAllGroups = function (indexes) {
            this.indexGroups().forEach(function (g) {
                g.indexes.removeAll(indexes);
            });

            this.indexGroups.remove(function (item) {
                return item.indexes().length === 0;
            });
        };

        indexes.prototype.unlockIndex = function (i) {
            this.updateIndexLockMode(i, "Unlock");
        };

        indexes.prototype.lockIndex = function (i) {
            this.updateIndexLockMode(i, "LockedIgnore");
        };

        indexes.prototype.lockErrorIndex = function (i) {
            this.updateIndexLockMode(i, "LockedError");
        };

        indexes.prototype.updateIndexLockMode = function (i, newLockMode) {
            var originalLockMode = i.lockMode();
            if (originalLockMode !== newLockMode) {
                i.lockMode(newLockMode);

                new saveIndexLockModeCommand(i, newLockMode, this.activeDatabase()).execute().fail(function () {
                    return i.lockMode(originalLockMode);
                });
            }
        };

        indexes.prototype.getAllIndexes = function () {
            var all = [];
            this.indexGroups().forEach(function (g) {
                return all.pushAll(g.indexes());
            });
            return all.distinct();
        };

        indexes.prototype.makeIndexPersistent = function (index) {
            new saveIndexAsPersistentCommand(index, this.activeDatabase()).execute();
        };
        return indexes;
    })(viewModelBase);

    
    return indexes;
});
