﻿var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "viewmodels/viewModelBase", "common/appUrl", "commands/getDatabaseStatsCommand", "commands/getIndexDefinitionCommand", "models/facet", "commands/queryFacetsCommand", "common/aceEditorBindingHandler", "common/pagedList"], function(require, exports, viewModelBase, appUrl, getDatabaseStatsCommand, getIndexDefinitionCommand, facet, queryFacetsCommand, aceEditorBindingHandler, pagedList) {
    var reporting = (function (_super) {
        __extends(reporting, _super);
        function reporting() {
            var _this = this;
            _super.call(this);
            this.selectedIndexName = ko.observable();
            this.selectedIndexLabel = ko.computed(function () {
                return _this.selectedIndexName() ? _this.selectedIndexName() : "[Select an index]";
            });
            this.indexNames = ko.observableArray();
            this.hasSelectedIndex = ko.computed(function () {
                return _this.selectedIndexName() && _this.selectedIndexName().length > 0;
            });
            this.editSelectedIndexUrl = ko.computed(function () {
                return _this.hasSelectedIndex() ? appUrl.forEditIndex(_this.selectedIndexName(), _this.activeDatabase()) : null;
            });
            this.availableFields = ko.observableArray();
            this.selectedField = ko.observable();
            this.selectedFieldLabel = ko.computed(function () {
                return _this.selectedField() ? _this.selectedField() : "Select a field";
            });
            this.addedValues = ko.observableArray();
            this.filter = ko.observable();
            this.hasFilter = ko.observable(false);
            this.reportResults = ko.observable();
            this.totalQueryResults = ko.computed(function () {
                return _this.reportResults() ? _this.reportResults().totalResultCount() : null;
            });
            this.queryDuration = ko.observable();
            this.appUrls = appUrl.forCurrentDatabase();
        }
        reporting.prototype.activate = function (indexToActivateOrNull) {
            var _this = this;
            _super.prototype.activate.call(this, indexToActivateOrNull);

            this.fetchIndexes().done(function () {
                return _this.selectInitialIndex(indexToActivateOrNull);
            });
            this.selectedIndexName.subscribe(function () {
                return _this.resetSelections();
            });

            aceEditorBindingHandler.install();
        };

        reporting.prototype.fetchIndexes = function () {
            var _this = this;
            return new getDatabaseStatsCommand(this.activeDatabase()).execute().done(function (results) {
                return _this.indexNames(results.Indexes.map(function (i) {
                    return i.PublicName;
                }));
            });
        };

        reporting.prototype.fetchIndexDefinition = function (indexName) {
            var _this = this;
            new getIndexDefinitionCommand(indexName, this.activeDatabase()).execute().done(function (dto) {
                return _this.availableFields(dto.Index.Fields);
            });
        };

        reporting.prototype.selectInitialIndex = function (indexToActivateOrNull) {
            if (indexToActivateOrNull && this.indexNames.contains(indexToActivateOrNull)) {
                this.setSelectedIndex(indexToActivateOrNull);
            } else if (this.indexNames().length > 0) {
                this.setSelectedIndex(this.indexNames.first());
            }
        };

        reporting.prototype.setSelectedIndex = function (indexName) {
            this.selectedIndexName(indexName);
            this.updateUrl(appUrl.forReporting(this.activeDatabase(), indexName));

            this.fetchIndexDefinition(indexName);
        };

        reporting.prototype.setSelectedField = function (fieldName) {
            this.selectedField(fieldName);

            this.addedValues().forEach(function (v) {
                return v.name = fieldName;
            });
        };

        reporting.prototype.resetSelections = function () {
            this.selectedField(null);
            this.addedValues([]);
            this.availableFields([]);
        };

        reporting.prototype.addValue = function (fieldName) {
            var val = facet.fromNameAndAggregation(this.selectedField(), fieldName);
            this.addedValues.push(val);
        };

        reporting.prototype.removeValue = function (val) {
            this.addedValues.remove(val);
        };

        reporting.prototype.runReport = function () {
            var _this = this;
            var selectedIndex = this.selectedIndexName();
            var filterQuery = this.hasFilter() ? this.filter() : null;
            var facets = this.addedValues().map(function (v) {
                return v.toDto();
            });
            var groupedFacets = [];
            facets.forEach(function (curFacet) {
                var foundFacet = groupedFacets.first(function (x) {
                    return x.AggregationField == curFacet.AggregationField;
                });

                if (foundFacet) {
                    foundFacet.Aggregation += curFacet.Aggregation;
                } else {
                    groupedFacets.push(curFacet);
                }
            });
            var db = this.activeDatabase();
            var resultsFetcher = function (skip, take) {
                return new queryFacetsCommand(selectedIndex, filterQuery, skip, take, groupedFacets, db).execute().done(function (resultSet) {
                    return _this.queryDuration(resultSet.additionalResultInfo);
                });
            };
            this.reportResults(new pagedList(resultsFetcher));
        };
        return reporting;
    })(viewModelBase);

    
    return reporting;
});
