var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "ace/ace", "viewmodels/viewModelBase", "commands/getDatabaseStatsCommand", "common/aceEditorBindingHandler", "models/scriptedIndexMap", "commands/getScriptedIndexesCommand", "commands/saveScriptedIndexesCommand"], function(require, exports, ace, viewModelBase, getDatabaseStatsCommand, aceEditorBindingHandler, scriptedIndexMap, getScriptedIndexesCommand, saveScriptedIndexesCommand) {
    var scriptedIndexes = (function (_super) {
        __extends(scriptedIndexes, _super);
        function scriptedIndexes() {
            _super.call(this);
            this.indexNames = ko.observableArray();
            this.selectedIndex = ko.observable();
            this.scrIndexes = ko.observable();
            this.scrIndex = ko.observable();
            this.isFirstLoad = true;

            aceEditorBindingHandler.install();
        }
        scriptedIndexes.prototype.canActivate = function (args) {
            var _this = this;
            var deferred = $.Deferred();
            var db = this.activeDatabase();
            if (db) {
                this.fetchAllIndexes(db).done(function () {
                    _this.fetchAllScriptedIndexes(db).done(function () {
                        deferred.resolve({ can: true });
                    });
                });
            }
            return deferred;
        };

        scriptedIndexes.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);

            this.isScriptIndexVisible = ko.computed(function () {
                return _this.scrIndex() && !_this.scrIndex().isMarkedToDelete();
            }, this);
            viewModelBase.dirtyFlag = new ko.DirtyFlag([this.scrIndexes().activeScriptedIndexes]);
            this.isSaveEnabled = ko.computed(function () {
                return viewModelBase.dirtyFlag().isDirty();
            });
        };

        scriptedIndexes.prototype.attached = function () {
            this.initializeScriptsTextboxes();
        };

        scriptedIndexes.prototype.fetchAllIndexes = function (db) {
            var _this = this;
            var deferred = $.Deferred();
            new getDatabaseStatsCommand(db).execute().done(function (results) {
                _this.performAllIndexesResult(results);
                deferred.resolve({ can: true });
            });
            return deferred;
        };

        scriptedIndexes.prototype.performAllIndexesResult = function (results) {
            this.indexNames(results.Indexes.map(function (i) {
                return i.PublicName;
            }));
        };

        scriptedIndexes.prototype.fetchAllScriptedIndexes = function (db) {
            var _this = this;
            var deferred = $.Deferred();
            new getScriptedIndexesCommand(db).execute().done(function (indexes) {
                _this.performAllScriptedIndexes(indexes);
                deferred.resolve({ can: true });
            });
            return deferred;
        };

        scriptedIndexes.prototype.performAllScriptedIndexes = function (indexes) {
            this.scrIndexes(new scriptedIndexMap(indexes));
            if (this.indexNames().length > 0) {
                this.setSelectedIndex(this.indexNames.first());
            }
        };

        scriptedIndexes.prototype.initializeScriptsTextboxes = function () {
            this.addIndexScriptHelpPopover();
            this.addDeleteScriptHelpPopover();
            this.startupAceEditor();
        };

        scriptedIndexes.prototype.addIndexScriptHelpPopover = function () {
            $("#indexScriptLabel").popover({
                html: true,
                trigger: 'hover',
                content: 'Index Scripts are written in JScript.<br/><br/>Example:</br><pre><span class="code-keyword">var</span> company = LoadDocument(<span class="code-keyword">this</span>.Company);<br/><span class="code-keyword">if</span>(company == null) <span class="code-keyword">return</span>;<br/>company.Orders = { Count: <span class="code-keyword">this</span>.Count, Total: <span class="code-keyword">this</span>.Total };<br/>PutDocument(<span class="code-keyword">this</span>.Company, company);</pre>'
            });
        };

        scriptedIndexes.prototype.addDeleteScriptHelpPopover = function () {
            $("#deleteScriptLabel").popover({
                html: true,
                trigger: 'hover',
                content: 'Index Scripts are written in JScript.<br/><br/>Example:</br><pre><span class="code-keyword">var</span> company = LoadDocument(<span class="code-keyword">this</span>.Company);<br/><span class="code-keyword">if</span> (company == null) <span class="code-keyword">return</span>;<br/><span class="code-keyword">delete</span> company.Orders;<br/>PutDocument(<span class="code-keyword">this</span>.Company, company);</pre>'
            });
        };

        scriptedIndexes.prototype.setSelectedIndex = function (indexName) {
            this.selectedIndex(indexName);
            this.getIndexForName(indexName);
        };

        scriptedIndexes.prototype.createNewScript = function () {
            this.scrIndexes().addEmptyIndex(this.selectedIndex());
            this.getIndexForName(this.selectedIndex());
        };

        scriptedIndexes.prototype.getIndexForName = function (indexName) {
            var index = this.scrIndexes().getIndex(indexName);
            this.scrIndex(index);

            if (index && !this.isFirstLoad) {
                this.initializeScriptsTextboxes();
            } else {
                this.isFirstLoad = false;
            }
        };

        scriptedIndexes.prototype.startupAceEditor = function () {
            var _this = this;
            if ($("#indexScriptEditor").length > 0) {
                ace.edit("indexScriptEditor").focus();
            }
            $("#indexScriptEditor").on('keyup', ".ace_text-input", function () {
                var value = ace.edit("indexScriptEditor").getSession().getValue();
                _this.scrIndex().indexScript(value);
            });
            $("#deleteScriptEditor").on('keyup', ".ace_text-input", function () {
                var value = ace.edit("deleteScriptEditor").getSession().getValue();
                _this.scrIndex().deleteScript(value);
            });
        };

        scriptedIndexes.prototype.saveChanges = function () {
            var _this = this;
            new saveScriptedIndexesCommand(this.scrIndexes(), this.activeDatabase()).execute().done(function (result) {
                _this.updateIndexes(result);

                viewModelBase.dirtyFlag().reset();
            });
        };

        scriptedIndexes.prototype.updateIndexes = function (serverIndexes) {
            var _this = this;
            this.scrIndexes().getIndexes().forEach(function (index) {
                var serverIndex = serverIndexes.first(function (k) {
                    return k.Key === index.getId();
                });
                if (serverIndex && !serverIndex.Deleted) {
                    index.__metadata.etag = serverIndex.Etag;
                    index.__metadata.lastModified = serverIndex.Metadata['Last-Modified'];
                } else if (serverIndex && serverIndex.Deleted) {
                    _this.scrIndexes().deleteMarkToDeletedIndex(serverIndex.Key);
                }
            });
        };

        scriptedIndexes.prototype.deleteScript = function () {
            this.scrIndexes().removeIndex(this.selectedIndex());
            this.getIndexForName(this.selectedIndex());
        };
        return scriptedIndexes;
    })(viewModelBase);

    
    return scriptedIndexes;
});
