var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "plugins/router", "common/appUrl", "viewmodels/viewModelBase", "commands/getDatabaseSettingsCommand"], function(require, exports, durandalRouter, appUrl, viewModelBase, getDatabaseSettingsCommand) {
    var settings = (function (_super) {
        __extends(settings, _super);
        function settings() {
            var _this = this;
            _super.call(this);
            this.router = null;
            this.bundleMap = { quotas: "Quotas", replication: "Replication", sqlreplication: "SQL Replication", versioning: "Versioning", periodicbackups: "Periodic Backup", scriptedindexresults: "Scripted Index", scriptedindex: "Scripted Index" };
            this.userDatabasePages = ko.observableArray(["Database Settings"]);
            this.systemDatabasePages = ["API Keys", "Windows Authentication"];

            this.appUrls = appUrl.forCurrentDatabase();

            this.isOnSystemDatabase = ko.computed(function () {
                return _this.activeDatabase() && _this.activeDatabase().isSystem;
            });
            this.isOnUserDatabase = ko.computed(function () {
                return _this.activeDatabase() && !_this.isOnSystemDatabase();
            });

            var apiKeyRoute = { route: 'databases/settings/apiKeys', moduleId: 'viewmodels/apiKeys', title: 'API Keys', nav: true, hash: appUrl.forApiKeys() };
            var windowsAuthRoute = { route: 'databases/settings/windowsAuth', moduleId: 'viewmodels/windowsAuth', title: 'Windows Authentication', nav: true, hash: appUrl.forWindowsAuth() };
            var databaseSettingsRoute = { route: ['databases/settings', 'databases/settings/databaseSettings'], moduleId: 'viewmodels/databaseSettings', title: 'Database Settings', nav: true, hash: appUrl.forCurrentDatabase().databaseSettings };

            var replicationsRoute = { route: 'databases/settings/replication', moduleId: 'viewmodels/replications', title: 'Replication', nav: true, hash: appUrl.forCurrentDatabase().replications };
            var sqlReplicationsRoute = { route: 'databases/settings/sqlReplication', moduleId: 'viewmodels/sqlReplications', title: 'SQL Replication', nav: true, hash: appUrl.forCurrentDatabase().sqlReplications };

            var periodicBackupRoute = { route: 'databases/settings/periodicBackups', moduleId: 'viewmodels/periodicBackup', title: 'Periodic Backup', nav: true, hash: appUrl.forCurrentDatabase().periodicBackup };
            var scriptedIndexesRoute = { route: 'databases/settings/scriptedIndex', moduleId: 'viewmodels/scriptedIndexes', title: 'Scripted Index', nav: true, hash: appUrl.forCurrentDatabase().scriptedIndexes };

            this.router = durandalRouter.createChildRouter().map([
                apiKeyRoute,
                windowsAuthRoute,
                databaseSettingsRoute,
                replicationsRoute,
                sqlReplicationsRoute,
                periodicBackupRoute,
                scriptedIndexesRoute
            ]).buildNavigationModel();

            this.router.guardRoute = function (instance, instruction) {
                return _this.getValidRoute(instance, instruction);
            };
        }
        settings.prototype.getValidRoute = function (instance, instruction) {
            var pathArr = instruction.fragment.split('/');
            var bundelName = pathArr[pathArr.length - 1].toLowerCase();
            var isLegalBundelName = (this.bundleMap[bundelName] != undefined);
            var isBundleExists = this.userDatabasePages.indexOf(this.bundleMap[bundelName]) >= 0;
            var isSystemDbOnlyPath = instruction.fragment.indexOf("windowsAuth") >= 0 || instruction.fragment.indexOf("apiKeys") >= 0 || instruction.fragment === "settings";
            var isUserDbOnlyPath = !isSystemDbOnlyPath;

            if ((isSystemDbOnlyPath && !this.activeDatabase().isSystem)) {
                return appUrl.forCurrentDatabase().databaseSettings();
            } else if (isUserDbOnlyPath && this.activeDatabase().isSystem) {
                return appUrl.forApiKeys();
            } else if (isUserDbOnlyPath && isLegalBundelName && !isBundleExists) {
                return appUrl.forCurrentDatabase().databaseSettings();
            }

            return true;
        };

        settings.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.userDatabasePages(["Database Settings"]);
            if (args) {
                var canActivateResult = $.Deferred();
                var db = this.activeDatabase();
                var self = this;
                new getDatabaseSettingsCommand(db).execute().done(function (document) {
                    var documentSettings = document.Settings["Raven/ActiveBundles"];
                    if (documentSettings != undefined) {
                        var arr = documentSettings.split(';');

                        for (var i = 0; i < arr.length; i++) {
                            var bundleName = self.bundleMap[arr[i].toLowerCase()];
                            if (bundleName != undefined) {
                                self.userDatabasePages.push(bundleName);
                            }
                        }
                    }

                    canActivateResult.resolve({ can: true });
                });
                return canActivateResult;
            } else {
                return $.Deferred().resolve({ can: true });
            }
        };

        settings.prototype.routeIsVisible = function (route) {
            var bundleTitle = route.title;

            if (this.isOnUserDatabase() && (this.userDatabasePages.indexOf(bundleTitle) !== -1)) {
                return true;
            }
            if (this.isOnSystemDatabase() && (this.systemDatabasePages.indexOf(bundleTitle) !== -1)) {
                return true;
            }

            return false;
        };
        return settings;
    })(viewModelBase);

    
    return settings;
});
