var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "models/sqlReplication", "viewmodels/viewModelBase", "common/aceEditorBindingHandler", "commands/getSqlReplicationsCommand", "commands/saveSqlReplicationsCommand", "commands/deleteDocumentsCommand", "common/appUrl"], function(require, exports, sqlReplication, viewModelBase, aceEditorBindingHandler, getSqlReplicationsCommand, saveSqlReplicationsCommand, deleteDocumentsCommand, appUrl) {
    var sqlReplications = (function (_super) {
        __extends(sqlReplications, _super);
        function sqlReplications() {
            _super.call(this);
            this.replications = ko.observableArray();
            this.isFirstload = ko.observable(true);
            this.lastIndex = ko.computed(function () {
                return this.isFirstload() ? -1 : this.replications().length - 1;
            }, this);
            this.loadedSqlReplications = [];
            this.itemNumber = function (index) {
                return index + 1;
            };

            aceEditorBindingHandler.install();
        }
        sqlReplications.prototype.canActivate = function (args) {
            var _this = this;
            var deferred = $.Deferred();
            var db = this.activeDatabase();
            if (db) {
                new getSqlReplicationsCommand(db).execute().done(function (results) {
                    for (var i = 0; i < results.length; i++) {
                        _this.loadedSqlReplications.push(results[i].getId());
                    }
                    _this.replications(results);

                    deferred.resolve({ can: true });
                }).fail(function () {
                    return deferred.resolve({ redirect: appUrl.forIndexes(_this.activeDatabase()) });
                });
            }
            return deferred;
        };

        sqlReplications.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);

            this.areAllSqlReplicationsValid = ko.computed(function () {
                return _this.replications().every(function (k) {
                    return k.isValid();
                });
            });
            viewModelBase.dirtyFlag = new ko.DirtyFlag([this.replications]);
            this.isSaveEnabled = ko.computed(function () {
                return viewModelBase.dirtyFlag().isDirty();
            });
        };

        sqlReplications.prototype.attached = function () {
            $(".script-label").popover({
                html: true,
                trigger: 'hover',
                content: 'Replication scripts use JScript.'
            });
        };

        sqlReplications.prototype.saveChanges = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                this.replications().forEach(function (r) {
                    return r.setIdFromName();
                });
                var deletedReplications = this.loadedSqlReplications.slice(0);
                var onScreenReplications = this.replications();

                for (var i = 0; i < onScreenReplications.length; i++) {
                    var replication = onScreenReplications[i];
                    var replicationId = replication.getId();
                    deletedReplications.remove(replicationId);

                    if (this.loadedSqlReplications.indexOf(replicationId) == -1) {
                        delete replication.__metadata.etag;
                        delete replication.__metadata.lastModified;
                    }
                }

                var deleteDeferred = this.deleteSqlReplications(deletedReplications, db);
                deleteDeferred.done(function () {
                    var saveDeferred = _this.saveSqlReplications(onScreenReplications, db);
                    saveDeferred.done(function () {
                        _this.updateLoadedSqlReplications();

                        viewModelBase.dirtyFlag().reset();
                    });
                });
            }
        };

        sqlReplications.prototype.deleteSqlReplications = function (deletedReplications, db) {
            var deleteDeferred = $.Deferred();

            if (deletedReplications.length > 0) {
                new deleteDocumentsCommand(deletedReplications, db).execute().done(function () {
                    deleteDeferred.resolve();
                });
            } else {
                deleteDeferred.resolve();
            }
            return deleteDeferred;
        };

        sqlReplications.prototype.saveSqlReplications = function (onScreenReplications, db) {
            var _this = this;
            var saveDeferred = $.Deferred();

            if (onScreenReplications.length > 0) {
                new saveSqlReplicationsCommand(this.replications(), db).execute().done(function (result) {
                    _this.updateKeys(result);
                    saveDeferred.resolve();
                });
            } else {
                saveDeferred.resolve();
            }
            return saveDeferred;
        };

        sqlReplications.prototype.updateLoadedSqlReplications = function () {
            this.loadedSqlReplications = [];
            var sqlReplications = this.replications();
            for (var i = 0; i < sqlReplications.length; i++) {
                this.loadedSqlReplications.push(sqlReplications[i].getId());
            }
        };

        sqlReplications.prototype.updateKeys = function (serverKeys) {
            this.replications().forEach(function (key) {
                var serverKey = serverKeys.first(function (k) {
                    return k.Key === key.getId();
                });
                if (serverKey) {
                    key.__metadata.etag = serverKey.Etag;
                    key.__metadata.lastModified = serverKey.Metadata['Last-Modified'];
                }
            });
        };

        sqlReplications.prototype.addNewSqlReplication = function () {
            this.isFirstload(false);
            this.replications.push(sqlReplication.empty());
        };

        sqlReplications.prototype.removeSqlReplication = function (repl) {
            this.replications.remove(repl);
        };
        return sqlReplications;
    })(viewModelBase);

    
    return sqlReplications;
});
