﻿var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "viewmodels/viewModelBase", "models/transformer", "commands/getTransformersCommand", "common/appUrl", "viewmodels/deleteTransformerConfirm", "durandal/app"], function(require, exports, viewModelBase, transformer, getTransformersCommand, appUrl, deleteTransformerConfirm, app) {
    var Transformers = (function (_super) {
        __extends(Transformers, _super);
        function Transformers() {
            _super.call(this);
            this.newTransformerUrl = appUrl.forCurrentDatabase().newTransformer;
            this.transformersGroups = ko.observableArray();
            this.containerSelector = "#transformersContainer";
        }
        Transformers.prototype.activate = function (args) {
            this.fetchTransformers();
            _super.prototype.activate.call(this, args);
        };

        Transformers.prototype.attached = function () {
            var _this = this;
            this.createKeyboardShortcut("Alt+N", function () {
                return _this.navigate(_this.newTransformerUrl());
            }, this.containerSelector);
            ko.postbox.publish("SetRawJSONUrl", appUrl.forTransformersRawData(this.activeDatabase()));
        };

        Transformers.prototype.modelPolling = function () {
            this.fetchTransformers();
        };

        Transformers.prototype.fetchTransformers = function () {
            var _this = this;
            new getTransformersCommand(this.activeDatabase()).execute().done(function (transformers) {
                transformers.map(function (curTransformer) {
                    return new transformer().initFromLoad(curTransformer);
                }).forEach(function (i) {
                    return _this.putTransformerIntoGroups(i);
                });
            });
        };

        Transformers.prototype.putTransformerIntoGroups = function (trans) {
            var groupName = trans.name().split("/")[0];
            var group = this.transformersGroups.first(function (g) {
                return g.entityName === groupName;
            });

            if (group) {
                var existingTrans = group.transformers.first(function (cur) {
                    return cur.name() == trans.name();
                });

                if (!existingTrans) {
                    group.transformers.push(trans);
                }
            } else {
                this.transformersGroups.push({ entityName: groupName, transformers: ko.observableArray([trans]) });
            }
        };

        Transformers.prototype.collapseAll = function () {
            $(".index-group-content").collapse('hide');
        };

        Transformers.prototype.expandAll = function () {
            $(".index-group-content").collapse('show');
        };

        Transformers.prototype.deleteAllTransformers = function () {
            var allTransformers;
            allTransformers = this.getAllTransformers();
            this.promptDeleteTransformers(allTransformers);
        };

        Transformers.prototype.getAllTransformers = function () {
            var all = [];
            this.transformersGroups().forEach(function (g) {
                return all.pushAll(g.transformers());
            });
            return all.distinct();
        };

        Transformers.prototype.deleteTransformer = function (transformerToDelete) {
            this.promptDeleteTransformers([transformerToDelete]);
        };

        Transformers.prototype.promptDeleteTransformers = function (transformers) {
            var _this = this;
            var db = this.activeDatabase();
            var deleteViewmodel = new deleteTransformerConfirm(transformers.map(function (i) {
                return i.name();
            }), db);
            deleteViewmodel.deleteTask.done(function () {
                return _this.removeTransformersFromAllGroups(transformers);
            });
            app.showDialog(deleteViewmodel);
        };

        Transformers.prototype.removeTransformersFromAllGroups = function (transformers) {
            this.transformersGroups().forEach(function (transGroup) {
                return transGroup.transformers.removeAll(transformers);
            });
            this.transformersGroups.remove(function (item) {
                return item.transformers().length === 0;
            });
        };
        return Transformers;
    })(viewModelBase);

    
    return Transformers;
});
